library dative_core;

import 'dart:async';
import 'package:flutter/material.dart';

import 'package:parse_server_sdk_flutter/parse_server_sdk_flutter.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'src/core/core_decoder.dart';
import 'src/utils/constants.dart';
import 'src/utils/core_utils.dart';
import 'src/utils/scheduler.dart';

class DativeCore with WidgetsBindingObserver {

  static String coreEndPoint = "core";
  static String installEndPoint = "install";

  final StreamController<void> _appResumedStreamController =
      StreamController<void>();

  Future<void> initialize({
    required String validationKey,
    required String serverUrl,
    required String liveQueryUrl,
    String? appName,
    String? appVersion,
    required ParseUser Function(String?, String?, String?,
            {ParseClient? client, bool? debug, String? sessionToken})?
        parseUserConstructor,
    required Map<String, ParseObject Function()>? subClassMap,
  }) async {
    SharedPreferences sharedPreferences = await SharedPreferences.getInstance();
    final model = await CoreScheduler(preferences: sharedPreferences, validationKey: validationKey).platformModel();
    await Parse().initialize(
      CoreUtils.validatedKey(model: model),
      CoreDecoder().decoded(model.key, decodedKey: serverUrl),
      clientKey: CoreUtils.clientKey(model: model),
      liveQueryUrl: CoreDecoder().decoded(model.key, decodedKey: liveQueryUrl),
      autoSendSessionId: true,
      coreStore: Constants.isWebPlatform()
          ? await CoreStoreSharedPreferences.getInstance()
          : await CoreStoreSembast.getInstance(
              password: CoreUtils.storageKey(model: model)),
      debug: Constants.isDebug,
      appName: appName,
      appPackageName: CoreUtils.appPackageName(model: model),
      appVersion: appVersion,
      appResumedStream: _appResumedStreamController.stream,
      parseUserConstructor: parseUserConstructor,
      registeredSubClassMap: subClassMap,
    );
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.resumed) {
      CoreScheduler().runTask();
      _appResumedStreamController.sink.add(null);
    }

    if (state == AppLifecycleState.paused) {
      CoreScheduler().runTask();
      _appResumedStreamController.close();
    }
  }
}
